==> fred-statementor-0.2.0.tar.gz: fred-statementor-0.2.0/statementor/main.py <==
"""Handles creation of record statements."""
from datetime import datetime
from io import BytesIO
from typing import Any, Dict, Tuple, Union, cast

from asgiref.sync import async_to_sync
from pytz import UTC
from regal import (Contact, ContactClient, ContactId, DomainClient, DomainId, Keyset, KeysetClient, KeysetId,
                   NssetClient, NssetId, ObjectEvents, RegistrarClient)
from typist import SecretaryClient

from .constants import Template

Timeout = Union[float, Tuple[float, float]]


class Statementor:
    """Handles creation of record statements."""

    def __init__(self, *, secretary_client: SecretaryClient, contact_client: ContactClient, domain_client: DomainClient,
                 keyset_client: KeysetClient, nsset_client: NssetClient, registrar_client: RegistrarClient):
        self.secretary_client = secretary_client
        self.contact_client = contact_client
        self.domain_client = domain_client
        self.keyset_client = keyset_client
        self.nsset_client = nsset_client
        self.registrar_client = registrar_client

    def _get_basic_context(self, private: bool) -> Dict[str, Any]:
        """Return shared context."""
        return {'private': private, 'timestamp': datetime.now(UTC).isoformat()}

    async def _get_registrar_context(self, handle: str) -> Dict[str, Any]:
        """Return context for the registrar."""
        registrar = await self.registrar_client.get_registrar_info(handle)

        return {'handle': registrar.registrar_handle, 'name': registrar.name, 'organization': registrar.organization}

    async def _get_linked_contact_context(self, contact_id: ContactId) -> Dict[str, Any]:
        """Return context for the linked contact."""
        contact = await self.contact_client.get_contact_info(contact_id)
        return {'handle': contact.contact_handle, 'name': contact.name, 'organization': contact.organization}

    def _get_events_context(self, events: ObjectEvents) -> Dict[str, Any]:
        """Return context for the object events."""
        context = {
            'registered': cast(datetime, events.registered.timestamp).isoformat(),
        }
        if events.updated and events.updated.timestamp:
            context['updated'] = events.updated.timestamp.isoformat()
        if events.transferred and events.transferred.timestamp:
            context['transferred'] = events.transferred.timestamp.isoformat()
        return context

    def _get_contact_base_context(self, contact: Contact) -> Dict[str, Any]:
        publish = contact.publish
        # Fix name of the address flag.
        if 'place' in publish:
            publish['address'] = publish.pop('place')
        if 'additional_identifier' in publish and contact.additional_identifier:
            publish[contact.additional_identifier.type] = publish.pop('additional_identifier')
        context: Dict[str, Any] = {
            'handle': contact.contact_handle,
            'name': contact.name,
            'organization': contact.organization,
            'vat_identification_number': contact.vat_identification_number,
            'emails': contact.emails,
            'notify_emails': contact.notify_emails,
            'telephone': contact.telephone,
            'fax': contact.fax,
            'publish': publish,
        }
        if contact.additional_identifier:
            context[contact.additional_identifier.type] = contact.additional_identifier.value
        if contact.place:
            context['address'] = {
                'streets': contact.place.street,
                'city': contact.place.city,
                'state_or_province': contact.place.state_or_province,
                'postal_code': contact.place.postal_code,
                'country_code': contact.place.country_code,
            }
        return context

    def _get_keyset_base_context(self, keyset: Keyset) -> Dict[str, Any]:
        dns_keys = []
        for dns_host in keyset.dns_keys:
            dns_key = {
                "flags": dns_host.flags,
                "protocol": dns_host.protocol,
                "algorithm": dns_host.alg,
                "key": dns_host.key,
            }
            dns_keys.append(dns_key)

        return {
            'handle': keyset.keyset_handle,
            'dns_keys': dns_keys,
        }

    async def _get_contact_context(self, contact_id: ContactId) -> Dict[str, Any]:
        """Return context for the contact."""
        contact = await self.contact_client.get_contact_info(contact_id)
        context = self._get_contact_base_context(contact)
        context['registrar'] = await self._get_registrar_context(cast(str, contact.sponsoring_registrar))
        context.update(self._get_events_context(cast(ObjectEvents, contact.events)))
        return context

    async def _get_domain_context(self, domain_id: DomainId) -> Dict[str, Any]:
        """Return context for the domain."""
        domain = await self.domain_client.get_domain_info(domain_id)
        contact = await self.contact_client.get_contact_info(cast(ContactId, domain.registrant))
        context: Dict[str, Any] = {
            'handle': domain.fqdn,
            'registrant': self._get_contact_base_context(contact),
            'registrar': await self._get_registrar_context(cast(str, domain.sponsoring_registrar)),
        }
        context['administrative_contacts'] = [await self._get_linked_contact_context(u)
                                              for u in domain.administrative_contacts]
        if domain.keyset:
            keyset = await self.keyset_client.get_keyset_info(domain.keyset)
            context['keyset'] = self._get_keyset_base_context(keyset)
            keyset_state = await self.keyset_client.get_keyset_state(domain.keyset)
            context['keyset']['flags'] = [k for k, v in keyset_state.items() if v]
        if domain.nsset:
            context['nsset'] = await self._get_nsset_context(domain.nsset)
            nsset_state = await self.nsset_client.get_nsset_state(domain.nsset)
            context['nsset']['flags'] = [k for k, v in nsset_state.items() if v]
        if domain.expires_at:
            context['expires_at'] = domain.expires_at.isoformat()
        context.update(self._get_events_context(cast(ObjectEvents, domain.events)))
        return context

    async def _get_keyset_context(self, keyset_id: KeysetId) -> Dict[str, Any]:
        """Return context for the keyset."""
        keyset = await self.keyset_client.get_keyset_info(keyset_id)
        context = self._get_keyset_base_context(keyset)
        context['registrar'] = await self._get_registrar_context(cast(str, keyset.sponsoring_registrar))
        context['technical_contacts'] = [await self._get_linked_contact_context(u) for u in keyset.technical_contacts]
        return context

    async def _get_nsset_context(self, nsset_id: NssetId) -> Dict[str, Any]:
        """Return context for the nsset."""
        nsset = await self.nsset_client.get_nsset_info(nsset_id)

        nameservers = []
        for dns_host in nsset.dns_hosts:
            ns = {
                'fqdn': dns_host.fqdn,
                'ip_addresses': [str(i) for i in dns_host.ip_addresses],
            }
            nameservers.append(ns)

        return {
            'handle': nsset.nsset_handle,
            'nameservers': nameservers,
            'technical_contacts': [await self._get_linked_contact_context(u) for u in nsset.technical_contacts],
            'registrar': await self._get_registrar_context(cast(str, nsset.sponsoring_registrar)),
        }

    def _render_pdf(self, template: Template, context: Dict[str, Any], *, timeout: Timeout = None) -> BytesIO:
        """Perform the rendering itself."""
        buff = BytesIO()
        kwargs = {}
        if timeout is not None:
            kwargs['timeout'] = timeout
        buff.write(self.secretary_client.render_pdf(template, context, **kwargs))
        buff.seek(0)
        return buff

    async def get_contact_statement(self, contact_id: Union[ContactId, str], *, private: bool = False,
                                    timeout: Timeout = None) -> BytesIO:
        """Return a PDF with a contact statement.

        Arguments:
            contact_id: The contact ID or handle.
            private: Whether the record is private.
            timeout: A timeout for statement rendering.

        Raises:
            ContactDoesNotExist: If the contact doesn't exist.
            ValueError: If the record statement couldn't be produced.
            RuntimeError: A transient error occured during the record statement creation.
        """
        context = self._get_basic_context(private)
        if not isinstance(contact_id, ContactId):
            # It's actually a handle.
            contact_id = await self.contact_client.get_contact_id(contact_id)
        context.update(await self._get_contact_context(contact_id))
        contact_state = await self.contact_client.get_contact_state(contact_id)
        context['flags'] = [k for k, v in contact_state.items() if v]
        return self._render_pdf(Template.CONTACT, context, timeout=timeout)

    async def get_domain_statement(self, domain_id: Union[DomainId, str], *, private: bool = False,
                                   timeout: Timeout = None) -> BytesIO:
        """Return a PDF with a domain statement.

        Arguments:
            domain_id: The domain ID or FQDN.
            private: Whether the record is private.
            timeout: A timeout for statement rendering.

        Raises:
            DomainDoesNotExist: If the domain doesn't exist.
            ValueError: If the record statement couldn't be produced.
            RuntimeError: A transient error occured during the record statement creation.
        """
        context = self._get_basic_context(private)
        if not isinstance(domain_id, DomainId):
            # It's actually a FQDN.
            domain_id = await self.domain_client.get_domain_id(domain_id)
        context.update(await self._get_domain_context(domain_id))
        domain_state = await self.domain_client.get_domain_state(domain_id)
        context['flags'] = [k for k, v in domain_state.items() if v]
        return self._render_pdf(Template.DOMAIN, context, timeout=timeout)

    async def get_keyset_statement(self, keyset_id: Union[KeysetId, str], *, private: bool = False,
                                   timeout: Timeout = None) -> BytesIO:
        """Return a PDF with a keyset statement.

        Arguments:
            keyset_id: The keyset ID or handle.
            private: Whether the record is private.
            timeout: A timeout for statement rendering.

        Raises:
            KeysetDoesNotExist: If the keyset doesn't exist.
            ValueError: If the record statement couldn't be produced.
            RuntimeError: A transient error occured during the record statement creation.
        """
        context = self._get_basic_context(private)
        if not isinstance(keyset_id, KeysetId):
            # It's actually a handle.
            keyset_id = await self.keyset_client.get_keyset_id(keyset_id)
        context.update(await self._get_keyset_context(keyset_id))
        keyset_state = await self.keyset_client.get_keyset_state(keyset_id)
        context['flags'] = [k for k, v in keyset_state.items() if v]
        return self._render_pdf(Template.KEYSET, context, timeout=timeout)

    async def get_nsset_statement(self, nsset_id: Union[NssetId, str], *, private: bool = False,
                                  timeout: Timeout = None) -> BytesIO:
        """Return a PDF with nsset statement.

        Arguments:
            nsset_id: The nsset ID or handle.
            private: Whether the record is private.
            timeout: A timeout for statement rendering.

        Raises:
            NssetDoesNotExist: If the nsset doesn't exist.
            ValueError: If the record statement couldn't be produced.
            RuntimeError: A transient error occured during the record statement creation.
        """
        context = self._get_basic_context(private)
        if not isinstance(nsset_id, NssetId):
            # It's actually a handle.
            nsset_id = await self.nsset_client.get_nsset_id(nsset_id)
        context.update(await self._get_nsset_context(nsset_id))
        nsset_state = await self.nsset_client.get_nsset_state(nsset_id)
        context['flags'] = [k for k, v in nsset_state.items() if v]
        return self._render_pdf(Template.NSSET, context, timeout=timeout)


class SyncStatementor(Statementor):
    """Handles creation of record statements with synchronous API."""

    def get_contact_statement(self, contact_id: Union[ContactId, str],  # type: ignore[override]
                              *, private: bool = False, timeout: Timeout = None) -> BytesIO:
        """Return a PDF with a contact statement.

        Arguments:
            contact_id: The contact ID or handle.
            private: Whether the record is private.
            timeout: A timeout for statement rendering.

        Raises:
            ContactDoesNotExist: If the contact doesn't exist.
            ValueError: If the record statement couldn't be produced.
            RuntimeError: A transient error occured during the record statement creation.
        """
        return cast(BytesIO, async_to_sync(super().get_contact_statement)(contact_id, private=private, timeout=timeout))

    def get_domain_statement(self, domain_id: Union[DomainId, str], *, private: bool = False,  # type: ignore[override]
                             timeout: Timeout = None) -> BytesIO:
        """Return a PDF with a domain statement.

        Arguments:
            domain_id: The domain ID or FQDN.
            private: Whether the record is private.
            timeout: A timeout for statement rendering.

        Raises:
            DomainDoesNotExist: If the domain doesn't exist.
            ValueError: If the record statement couldn't be produced.
            RuntimeError: A transient error occured during the record statement creation.
        """
        return cast(BytesIO, async_to_sync(super().get_domain_statement)(domain_id, private=private, timeout=timeout))

    def get_keyset_statement(self, keyset_id: Union[KeysetId, str], *, private: bool = False,  # type: ignore[override]
                             timeout: Timeout = None) -> BytesIO:
        """Return a PDF with a keyset statement.

        Arguments:
            keyset_id: The keyset ID or handle.
            private: Whether the record is private.
            timeout: A timeout for statement rendering.

        Raises:
            KeysetDoesNotExist: If the keyset doesn't exist.
            ValueError: If the record statement couldn't be produced.
            RuntimeError: A transient error occured during the record statement creation.
        """
        return cast(BytesIO, async_to_sync(super().get_keyset_statement)(keyset_id, private=private, timeout=timeout))

    def get_nsset_statement(self, nsset_id: Union[NssetId, str], *, private: bool = False,  # type: ignore[override]
                            timeout: Timeout = None) -> BytesIO:
        """Return a PDF with nsset statement.

        Arguments:
            nsset_id: The nsset ID or handle.
            private: Whether the record is private.
            timeout: A timeout for statement rendering.

        Raises:
            NssetDoesNotExist: If the nsset doesn't exist.
            ValueError: If the record statement couldn't be produced.
            RuntimeError: A transient error occured during the record statement creation.
        """
        return cast(BytesIO, async_to_sync(super().get_nsset_statement)(nsset_id, private=private, timeout=timeout))

==> fred-statementor-1.0.0.tar.gz: fred-statementor-1.0.0/statementor/main.py <==
"""Handles creation of record statements."""
from datetime import datetime
from io import BytesIO
from typing import Any, Dict, Optional, Tuple, Union, cast

from asgiref.sync import async_to_sync
from pytz import UTC
from regal import (Contact, ContactClient, ContactId, ContactRef, DomainClient, DomainId, Keyset, KeysetClient,
                   KeysetId, NssetClient, NssetId, ObjectEvents, RegistrarClient)
from typist import SecretaryClient

from .constants import Template

Timeout = Union[float, Tuple[float, float]]


class Statementor:
    """Handles creation of record statements."""

    def __init__(self, *, secretary_client: SecretaryClient, contact_client: ContactClient, domain_client: DomainClient,
                 keyset_client: KeysetClient, nsset_client: NssetClient, registrar_client: RegistrarClient):
        self.secretary_client = secretary_client
        self.contact_client = contact_client
        self.domain_client = domain_client
        self.keyset_client = keyset_client
        self.nsset_client = nsset_client
        self.registrar_client = registrar_client

    def _get_basic_context(self, private: bool) -> Dict[str, Any]:
        """Return shared context."""
        return {'private': private, 'timestamp': datetime.now(UTC).isoformat()}

    async def _get_registrar_context(self, handle: str) -> Dict[str, Any]:
        """Return context for the registrar."""
        registrar = await self.registrar_client.get_registrar_info(handle)

        return {'handle': registrar.registrar_handle, 'name': registrar.name, 'organization': registrar.organization}

    async def _get_linked_contact_context(self, contact_id: ContactId) -> Dict[str, Any]:
        """Return context for the linked contact."""
        contact = await self.contact_client.get_contact_info(contact_id)
        return {'handle': contact.contact_handle, 'name': contact.name, 'organization': contact.organization}

    def _get_events_context(self, events: ObjectEvents) -> Dict[str, Any]:
        """Return context for the object events."""
        context = {
            'registered': cast(datetime, events.registered.timestamp).isoformat(),
        }
        if events.updated and events.updated.timestamp:
            context['updated'] = events.updated.timestamp.isoformat()
        if events.transferred and events.transferred.timestamp:
            context['transferred'] = events.transferred.timestamp.isoformat()
        return context

    def _get_contact_base_context(self, contact: Contact) -> Dict[str, Any]:
        publish = contact.publish
        # Fix name of the address flag.
        if 'place' in publish:
            publish['address'] = publish.pop('place')
        if 'additional_identifier' in publish and contact.additional_identifier:
            publish[contact.additional_identifier.type] = publish.pop('additional_identifier')
        context: Dict[str, Any] = {
            'handle': contact.contact_handle,
            'name': contact.name,
            'organization': contact.organization,
            'vat_identification_number': contact.vat_identification_number,
            'emails': contact.emails,
            'notify_emails': contact.notify_emails,
            'telephone': contact.telephone,
            'fax': contact.fax,
            'publish': publish,
        }
        if contact.additional_identifier:
            context[contact.additional_identifier.type] = contact.additional_identifier.value
        if contact.place:
            context['address'] = {
                'streets': contact.place.street,
                'city': contact.place.city,
                'state_or_province': contact.place.state_or_province,
                'postal_code': contact.place.postal_code,
                'country_code': contact.place.country_code,
            }
        return context

    def _get_keyset_base_context(self, keyset: Keyset) -> Dict[str, Any]:
        dns_keys = []
        for dns_host in keyset.dns_keys:
            dns_key = {
                "flags": dns_host.flags,
                "protocol": dns_host.protocol,
                "algorithm": dns_host.alg,
                "key": dns_host.key,
            }
            dns_keys.append(dns_key)

        return {
            'handle': keyset.keyset_handle,
            'dns_keys': dns_keys,
        }

    async def _get_contact_context(self, contact_id: ContactId) -> Dict[str, Any]:
        """Return context for the contact."""
        contact = await self.contact_client.get_contact_info(contact_id)
        context = self._get_contact_base_context(contact)
        context['registrar'] = await self._get_registrar_context(cast(str, contact.sponsoring_registrar))
        context.update(self._get_events_context(cast(ObjectEvents, contact.events)))
        return context

    async def _get_domain_context(self, domain_id: DomainId) -> Dict[str, Any]:
        """Return context for the domain."""
        domain = await self.domain_client.get_domain_info(domain_id)
        contact = await self.contact_client.get_contact_info(cast(ContactRef, domain.registrant).id)
        context: Dict[str, Any] = {
            'handle': domain.fqdn,
            'registrant': self._get_contact_base_context(contact),
            'registrar': await self._get_registrar_context(cast(str, domain.sponsoring_registrar)),
        }
        context['administrative_contacts'] = [await self._get_linked_contact_context(u.id)
                                              for u in domain.administrative_contacts]
        if domain.keyset:
            keyset = await self.keyset_client.get_keyset_info(domain.keyset.id)
            context['keyset'] = self._get_keyset_base_context(keyset)
            keyset_state = await self.keyset_client.get_keyset_state(domain.keyset.id)
            context['keyset']['flags'] = [k for k, v in keyset_state.items() if v]
        if domain.nsset:
            context['nsset'] = await self._get_nsset_context(domain.nsset.id)
            nsset_state = await self.nsset_client.get_nsset_state(domain.nsset.id)
            context['nsset']['flags'] = [k for k, v in nsset_state.items() if v]
        if domain.expires_at:
            context['expires_at'] = domain.expires_at.date().isoformat()
        context.update(self._get_events_context(cast(ObjectEvents, domain.events)))
        return context

    async def _get_keyset_context(self, keyset_id: KeysetId) -> Dict[str, Any]:
        """Return context for the keyset."""
        keyset = await self.keyset_client.get_keyset_info(keyset_id)
        context = self._get_keyset_base_context(keyset)
        context['registrar'] = await self._get_registrar_context(cast(str, keyset.sponsoring_registrar))
        context['technical_contacts'] = [await self._get_linked_contact_context(u.id)
                                         for u in keyset.technical_contacts]
        return context

    async def _get_nsset_context(self, nsset_id: NssetId) -> Dict[str, Any]:
        """Return context for the nsset."""
        nsset = await self.nsset_client.get_nsset_info(nsset_id)

        nameservers = []
        for dns_host in nsset.dns_hosts:
            ns = {
                'fqdn': dns_host.fqdn,
                'ip_addresses': [str(i) for i in dns_host.ip_addresses],
            }
            nameservers.append(ns)

        return {
            'handle': nsset.nsset_handle,
            'nameservers': nameservers,
            'technical_contacts': [await self._get_linked_contact_context(u.id) for u in nsset.technical_contacts],
            'registrar': await self._get_registrar_context(cast(str, nsset.sponsoring_registrar)),
        }

    def _render_pdf(self, template: Template, context: Dict[str, Any], *, timeout: Optional[Timeout] = None) -> BytesIO:
        """Perform the rendering itself."""
        buff = BytesIO()
        kwargs = {}
        if timeout is not None:
            kwargs['timeout'] = timeout
        buff.write(self.secretary_client.render_pdf(template, context, **kwargs))
        buff.seek(0)
        return buff

    async def get_contact_statement(self, contact_id: Union[ContactId, str], *, private: bool = False,
                                    timeout: Optional[Timeout] = None) -> BytesIO:
        """Return a PDF with a contact statement.

        Arguments:
            contact_id: The contact ID or handle.
            private: Whether the record is private.
            timeout: A timeout for statement rendering.

        Raises:
            ContactDoesNotExist: If the contact doesn't exist.
            ValueError: If the record statement couldn't be produced.
            RuntimeError: A transient error occured during the record statement creation.
        """
        context = self._get_basic_context(private)
        if not isinstance(contact_id, ContactId):
            # It's actually a handle.
            contact_id = await self.contact_client.get_contact_id(contact_id)
        context.update(await self._get_contact_context(contact_id))
        contact_state = await self.contact_client.get_contact_state(contact_id)
        context['flags'] = [k for k, v in contact_state.items() if v]
        return self._render_pdf(Template.CONTACT, context, timeout=timeout)

    async def get_domain_statement(self, domain_id: Union[DomainId, str], *, private: bool = False,
                                   timeout: Optional[Timeout] = None) -> BytesIO:
        """Return a PDF with a domain statement.

        Arguments:
            domain_id: The domain ID or FQDN.
            private: Whether the record is private.
            timeout: A timeout for statement rendering.

        Raises:
            DomainDoesNotExist: If the domain doesn't exist.
            ValueError: If the record statement couldn't be produced.
            RuntimeError: A transient error occured during the record statement creation.
        """
        context = self._get_basic_context(private)
        if not isinstance(domain_id, DomainId):
            # It's actually a FQDN.
            domain_id = await self.domain_client.get_domain_id(domain_id)
        context.update(await self._get_domain_context(domain_id))
        domain_state = await self.domain_client.get_domain_state(domain_id)
        context['flags'] = [k for k, v in domain_state.items() if v]
        return self._render_pdf(Template.DOMAIN, context, timeout=timeout)

    async def get_keyset_statement(self, keyset_id: Union[KeysetId, str], *, private: bool = False,
                                   timeout: Optional[Timeout] = None) -> BytesIO:
        """Return a PDF with a keyset statement.

        Arguments:
            keyset_id: The keyset ID or handle.
            private: Whether the record is private.
            timeout: A timeout for statement rendering.

        Raises:
            KeysetDoesNotExist: If the keyset doesn't exist.
            ValueError: If the record statement couldn't be produced.
            RuntimeError: A transient error occured during the record statement creation.
        """
        context = self._get_basic_context(private)
        if not isinstance(keyset_id, KeysetId):
            # It's actually a handle.
            keyset_id = await self.keyset_client.get_keyset_id(keyset_id)
        context.update(await self._get_keyset_context(keyset_id))
        keyset_state = await self.keyset_client.get_keyset_state(keyset_id)
        context['flags'] = [k for k, v in keyset_state.items() if v]
        return self._render_pdf(Template.KEYSET, context, timeout=timeout)

    async def get_nsset_statement(self, nsset_id: Union[NssetId, str], *, private: bool = False,
                                  timeout: Optional[Timeout] = None) -> BytesIO:
        """Return a PDF with nsset statement.

        Arguments:
            nsset_id: The nsset ID or handle.
            private: Whether the record is private.
            timeout: A timeout for statement rendering.

        Raises:
            NssetDoesNotExist: If the nsset doesn't exist.
            ValueError: If the record statement couldn't be produced.
            RuntimeError: A transient error occured during the record statement creation.
        """
        context = self._get_basic_context(private)
        if not isinstance(nsset_id, NssetId):
            # It's actually a handle.
            nsset_id = await self.nsset_client.get_nsset_id(nsset_id)
        context.update(await self._get_nsset_context(nsset_id))
        nsset_state = await self.nsset_client.get_nsset_state(nsset_id)
        context['flags'] = [k for k, v in nsset_state.items() if v]
        return self._render_pdf(Template.NSSET, context, timeout=timeout)


class SyncStatementor(Statementor):
    """Handles creation of record statements with synchronous API."""

    def get_contact_statement(self, contact_id: Union[ContactId, str],  # type: ignore[override]
                              *, private: bool = False, timeout: Optional[Timeout] = None) -> BytesIO:
        """Return a PDF with a contact statement.

        Arguments:
            contact_id: The contact ID or handle.
            private: Whether the record is private.
            timeout: A timeout for statement rendering.

        Raises:
            ContactDoesNotExist: If the contact doesn't exist.
            ValueError: If the record statement couldn't be produced.
            RuntimeError: A transient error occured during the record statement creation.
        """
        return cast(BytesIO, async_to_sync(super().get_contact_statement)(contact_id, private=private, timeout=timeout))

    def get_domain_statement(self, domain_id: Union[DomainId, str], *, private: bool = False,  # type: ignore[override]
                             timeout: Optional[Timeout] = None) -> BytesIO:
        """Return a PDF with a domain statement.

        Arguments:
            domain_id: The domain ID or FQDN.
            private: Whether the record is private.
            timeout: A timeout for statement rendering.

        Raises:
            DomainDoesNotExist: If the domain doesn't exist.
            ValueError: If the record statement couldn't be produced.
            RuntimeError: A transient error occured during the record statement creation.
        """
        return cast(BytesIO, async_to_sync(super().get_domain_statement)(domain_id, private=private, timeout=timeout))

    def get_keyset_statement(self, keyset_id: Union[KeysetId, str], *, private: bool = False,  # type: ignore[override]
                             timeout: Optional[Timeout] = None) -> BytesIO:
        """Return a PDF with a keyset statement.

        Arguments:
            keyset_id: The keyset ID or handle.
            private: Whether the record is private.
            timeout: A timeout for statement rendering.

        Raises:
            KeysetDoesNotExist: If the keyset doesn't exist.
            ValueError: If the record statement couldn't be produced.
            RuntimeError: A transient error occured during the record statement creation.
        """
        return cast(BytesIO, async_to_sync(super().get_keyset_statement)(keyset_id, private=private, timeout=timeout))

    def get_nsset_statement(self, nsset_id: Union[NssetId, str], *, private: bool = False,  # type: ignore[override]
                            timeout: Optional[Timeout] = None) -> BytesIO:
        """Return a PDF with nsset statement.

        Arguments:
            nsset_id: The nsset ID or handle.
            private: Whether the record is private.
            timeout: A timeout for statement rendering.

        Raises:
            NssetDoesNotExist: If the nsset doesn't exist.
            ValueError: If the record statement couldn't be produced.
            RuntimeError: A transient error occured during the record statement creation.
        """
        return cast(BytesIO, async_to_sync(super().get_nsset_statement)(nsset_id, private=private, timeout=timeout))
